Library UNISIM;
use UNISIM.vcomponents.all;
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

entity DAC_DDR_IO is Port(
	refclk200: in std_logic;
	delay_locked: out std_logic;
	IO_Reset: in std_logic:='0';
	--- DAC IO ---
	D: out std_logic_vector(15 downto 0);
	IQSEL: out std_logic;
	DCO: in std_logic);
end DAC_DDR_IO;

architecture Behavioral of DAC_DDR_IO is

component selectio_wiz_0
generic
 (-- width of the data for the system
  SYS_W       : integer := 16;
  -- width of the data for the device
  DEV_W       : integer := 32);
port
 (
  -- From the device out to the system
  data_out_from_device    : in    std_logic_vector(DEV_W-1 downto 0);
  data_out_to_pins        : out   std_logic_vector(SYS_W-1 downto 0);
  clk_to_pins             : out std_logic;

  delay_locked            : out   std_logic;                    -- Locked signal from IDELAYCTRL
  ref_clock               : in    std_logic;                    -- Reference Clock for IDELAYCTRL. Has to come from BUFG.
 
-- Clock and reset signals
  clk_in                  : in    std_logic;                    -- Single ended Fast clock from IOB
  clk_out                 : out   std_logic;
  clk_reset               : in    std_logic;                    -- Reset signal for Clock circuit
  io_reset                : in    std_logic);                   -- Reset signal for IO circuit
end component;

signal DAC1_D: std_logic_vector(15 downto 0):=x"0000";
signal DAC2_D: std_logic_vector(15 downto 0):=x"0000";
signal DAC1_counter: unsigned(15 downto 0):=x"0000";
signal DAC2_counter: unsigned(15 downto 0):=x"0000";
signal DCO_intern: std_logic:='0';

begin

IO : selectio_wiz_0 port map( 
	data_out_from_device => DAC1_D & DAC2_D,
	data_out_to_pins => D,
	clk_to_pins => IQSEL,
	delay_locked => delay_locked,                      
	ref_clock => refclk200,                         
	clk_in => DCO,                            
	clk_out => DCO_intern,
	clk_reset => IO_Reset,
	io_reset => IO_Reset);

process begin
	wait until rising_edge(DCO_intern);
	-- DAC1
	DAC1_counter <= DAC1_counter +1;
	DAC1_D <= std_logic_vector(DAC1_counter);
	-- DAC2
	DAC2_counter <= DAC2_counter -1;
	DAC2_D <= std_logic_vector(DAC2_counter);
end process;

end Behavioral;